using UnityEngine;
using System.Collections;

public class GameUI : MonoBehaviour {

	private static GameUI _instance;
	public static GameUI instance {
		get {
			return _instance;
		}
	}
	
	public Camera guiCamera;
	
	public UIPanel dialogPanel;
	public PackedSprite dialogWindow;
	public SpriteText dialogText;
	public SpriteText complete;
	public SpriteText paint;
	
	public UIButton blueButton;
	public UIButton yellowButton;
	public UIButton redButton;
	public UIButton purpleButton;
	public UIButton greenButton;
	public UIButton orangeButton;
	
	private string currentText = "";
	private int maxTextLines = 4;
	private float secondsPerLetter = 0.08f;
	private bool doingDialog = false;
	
	void Awake()
	{
		_instance = this;
	}
	
	void OnDestroy()
	{
		_instance = null;
	}
	
	void Start()
	{
		dialogPanel.gameObject.SetActiveRecursively(false);
		UpdateButtons();
	}
	
	void Update()
	{
		complete.Text = "Complete: " + (ColorBlock.PercentColored() * 100).ToString("0.0") + "%";
		paint.Text = "Health: " + Mathf.FloorToInt(PlayerController.instance.stats.health);
		SetAllButtonsEnabled();
	}
	
	public void UpdateButtons()
	{
		
		if (Game.GetFlag("YellowRescued")) {
			yellowButton.Hide(false);
		} else {
			yellowButton.Hide(true);
		}
		
		if (Game.GetFlag("RedRescued")) {
			redButton.Hide(false);
		} else {
			redButton.Hide(true);
		}
		
		if (Game.GetFlag("PurpleRescued")) {
			purpleButton.Hide(false);
		} else {
			purpleButton.Hide(true);
		}
		
		if (Game.GetFlag("GreenRescued")) {
			greenButton.Hide(false);
		} else {
			greenButton.Hide(true);
		}
		
		if (Game.GetFlag("OrangeRescued")) {
			orangeButton.Hide(false);
		} else {
			orangeButton.Hide(true);
		}
		
	}
	
	public void SetAllButtonsEnabled()
	{
		blueButton.SetState(0);
		
		yellowButton.SetState(0);
		
		redButton.SetState(0);
		
		purpleButton.SetState(0);
		
		greenButton.SetState(0);
		
		orangeButton.SetState(0);
		
		switch (PlayerController.instance.weaponColor) {
		
			case PlayerColor.Blue:
			blueButton.SetState(3);
			break;
			
			case PlayerColor.Yellow:
			yellowButton.SetState(3);
			break;
			
			case PlayerColor.Red:
			redButton.SetState(3);
			break;
			
			case PlayerColor.Purple:
			purpleButton.SetState(3);
			break;
			
			case PlayerColor.Green:
			greenButton.SetState(3);
			break;
			
			case PlayerColor.Orange:
			orangeButton.SetState(3);
			break;
			
		}
		
	}
	
	public void Blue()
	{
		PlayerController.instance.weaponColor = PlayerColor.Blue;
	}
	
	public void Red()
	{
		PlayerController.instance.weaponColor = PlayerColor.Red;
	}
	
	public void Yellow()
	{
		PlayerController.instance.weaponColor = PlayerColor.Yellow;
	}
	
	public void Purple()
	{
		PlayerController.instance.weaponColor = PlayerColor.Purple;
	}
	
	public void Green()
	{
		PlayerController.instance.weaponColor = PlayerColor.Green;
	}
	
	public void Orange()
	{
		PlayerController.instance.weaponColor = PlayerColor.Orange;
	}
	
	public void Dialog(string text, bool justText)
	{
		if (doingDialog) {
			return;
		}
		
		if (justText) {
			StartCoroutine(JustText(text));
		} else {
			StartCoroutine(WaitDialog(text));
		}
	}
	
	public IEnumerator JustText(string text)
	{
		dialogWindow.Hide(true);
		yield return StartCoroutine(WaitDialog(text));
		dialogWindow.Hide(false);	
	}
	
	public bool DoingDialog()
	{
		return doingDialog;
	}
	
	public void Dialog(string text)
	{
		if (doingDialog) {
			return;
		}
		
		StartCoroutine(WaitDialog(text));	
	}
	
	public IEnumerator WaitDialog(string text)
	{
		doingDialog = true;
		dialogPanel.BringIn();
		
		//Split by spaces
		string[] words = text.Split(" "[0]);
		
		currentText = "";
				
		for (int i = 0; i < words.Length; i++) {

			//Check the next word
			string word = words[i];
			
			string temp;
			
			if (currentText.Length == 0) {
				temp = word;
			} else {
				//Add a space to the front of the word
				temp = currentText + " " + word;
			}
			
			//Set to temp just for line test
			dialogText.Text = temp;
			
			if (Input.GetKeyDown(KeyCode.Escape)) {
				dialogPanel.Dismiss();
				doingDialog = false;
				yield break;
			}
			
			if (dialogText.GetDisplayLineCount() > maxTextLines) {
				
				dialogText.Text = currentText;
				
				while (!Input.anyKeyDown) {
					yield return null;
				}
				currentText = "";
				
			} else {
				
				dialogText.Text = currentText;
			
			}
			
			//Add a space to front of the word if we haven't cleared the text and it's not the first word
			if (currentText.Length > 0) {
				word = " " + word;
			}
			
			//Slowly Add Word to current text
			yield return StartCoroutine(AddWordToText(word));
			
			dialogText.Text = currentText;
						
		}
		
		while (!Input.anyKeyDown) {
			yield return null;
		}
		
		dialogPanel.Dismiss();
		doingDialog = false;
	}
	
	private IEnumerator AddWordToText(string text)
	{
	
		bool done = false;
		int currentPosition = 0;
		bool insideTag = false;
		
		while (!done) {
			
			//Draw next letter
			string newText = text.Substring(currentPosition, 1);
			
			if (newText == "\n") {
				while (!Input.anyKeyDown) {
					yield return null;
				}
			}
			
			if (!insideTag) {
				if (newText == "[") {
					insideTag = true;
				}
			} else {
				if (newText == "]") {
					insideTag = false;
				}
			}
			
			currentText += newText;
			currentPosition += 1;
			
			dialogText.Text = currentText;
			
			if (Input.GetKeyDown(KeyCode.Escape)) {
				dialogText.Text = currentText + text.Substring(currentPosition);
				yield break;
			}
			
			//Check if we're done
			if (currentPosition >= text.Length) {
				currentPosition = text.Length;
				done = true;
			} else {
				//Wait for next letter if we're not inside a tag
				if (!insideTag) {
					yield return new WaitForSeconds(secondsPerLetter);
				}
				
			}
			
		}
	
	}
	
}
