using UnityEngine;
using System.Collections;

public class Monochrome : EnemyController {

	public ColorBackground background;
	public PlayerColor color;
	public Particle particle;
	
	private float attackCooldown = 0f;
	
	private bool noColorYet = true;
	private bool attacking = false;
	
	private float colorChange = 0;
	
	public void Update()
	{
		
		if (colorChange > 0) {
			colorChange -= Time.deltaTime;
		}
		
		CheckImmunity();
		CheckPhysics();
		
		if (behaviorEnabled) {
			if (alwaysActive || renderer.isVisible) {
				CheckDirection();
			}
		}
				
		SnapToGrid();
		
		if (attacking) {
			return;
		}
		
		if (attackCooldown < 0 && behaviorEnabled) {
			Attack();
		} else {
			attackCooldown -= Time.deltaTime;
		}
		
	}
	
	public override void CheckDirection()
	{
		
		if (PlayerController.instance == null) {
			return;
		}
		
		float playerX = PlayerController.instance.x;
		float dist = Mathf.Abs(playerX - this.x);
		
		/*if (stationary || attacking) {
			walking = false;
		} else if (dist > 2f) {
			walking = true;
		} else {
			walking = false;
		}*/
		
		if (playerX < this.x) {
			direction = -1f;
		} else if (playerX > this.x) {
			direction = 1f;
		}
		
		if (flipped && direction == -1.0f) {
			sprite.FlipX();
			flipped = !flipped;
		} else if (!flipped && direction == 1.0f) {
			sprite.FlipX();
			flipped = !flipped;
		}
		
	}
	
	void Attack()
	{
		
		//Attack speed is in attacks per second
		attackCooldown = 1f / stats.attackSpeed;
		
		//Attacks: Jump at player, Shoot Projectiles, Change Color
		
		int roll = Random.Range(0, 100);
		
		if (roll < 30) {
			StartCoroutine(DoCharge());
		}else if (roll < 55) {
			//Jump
			if (!IsGrounded()) {
				StartCoroutine(DoShoot());
			} else {
				StartCoroutine(DoJump());
			}
		} else if (roll < 80) {
			//Shoot
			StartCoroutine(DoShoot());
		} else if (roll < 100) {
			//Change Colors
			if (colorChange > 0) {
				StartCoroutine(DoShoot());
			} else {
				StartCoroutine(DoNewColor());
			}
		}
		
		//Projectile projectile = Instantiate(projectilePrefab, transform.position + Vector3.up * 8, Quaternion.identity) as Projectile;
		//projectile.direction = new Vector3(direction, 0, 0);
		
	}
	
	IEnumerator DoNewColor()
	{
		Debug.Log("New Color");
		attacking = true;
		
		sprite.Play();
		
		PlayerColor[] colors = {PlayerColor.Blue,PlayerColor.Green,PlayerColor.Orange,PlayerColor.Purple,PlayerColor.Red,PlayerColor.Yellow};
		
		color = colors[Random.Range(0, colors.Length)];
		
		if (background != null) {
			background.SetColor(color);
		}
		
		yield return new WaitForSeconds(2f);
	
		sprite.Stop();
		
		attacking = false;
		
		colorChange = 5f;
		Debug.Log("New Color Done");
	}
	
	IEnumerator DoCharge()
	{
		Debug.Log("Charge");
		attacking = true;
		
		sprite.Play();
		
		float temp = this.speed;
		this.speed *= 1.5f;
		
		walking = true;
		
		//Charge for 2 seconds
		yield return new WaitForSeconds(2f);
		
		walking = false;
		
		this.speed = temp;
		
		sprite.Stop();
		
		attacking = false;
		
		Debug.Log("Charge Done");
		
	}
	
	IEnumerator DoJump()
	{
		Debug.Log("Jump");
		attacking = true;
		
		sprite.Play();
		
		velocity.y += 100f;
		velocity.x += 40f * direction;
		
		while (!IsGrounded()) {
			yield return new WaitForSeconds(0.5f);
		}
		
		sprite.Stop();
		
		attacking = false;
		Debug.Log("Jump Done");
		
	}
	
	IEnumerator DoShoot()
	{
		Debug.Log("Shoot");
		Vector3 particleVelocity = new Vector3(50f, 32f, 0f);
		
		//Shoot out 5 particles at increasing velocities
		for (int i = 0; i < 5; i++) {
			
			Particle newParticle = Instantiate(particle, transform.position + new Vector3(direction*6,16f,0f), Quaternion.identity) as Particle;
			
			newParticle.velocity = new Vector3(particleVelocity.x * direction, particleVelocity.y * (i + 1), 0);
			newParticle.color = Game.GetColor(color);
			newParticle.dieOnGroundHit = true;
			
			yield return new WaitForSeconds(0.25f);
			
		}
		
		Debug.Log("Shoot Done");
	}
	
	public override void Die()
	{
		
		if (messageDelegate != null && deathMessage != "") {
			messageDelegate.SendMessage(deathMessage);
		}
		
		behaviorEnabled = false;
		
	}
	
	public override void OnCollisionEnter(Collision collision)
	{
		
		if (!behaviorEnabled) {
			return;
		}
		
		if (collision.gameObject.layer == LayerMask.NameToLayer("Enemy")) {
			return;
		}
		
		Particle particle = collision.gameObject.GetComponent<Particle>();
		
		if (particle == null) {
			return;
		}
		
		//Can only be hurt by current color
		if (particle.color != Game.GetColor(color)) {
			return;
		}
		
		Damage damage = collision.gameObject.GetComponent<Damage>();
		
		if (damage != null && immunity <= 0) {			
			this.ApplyDamage(damage.attack, damage.knockback);
			immunity = 0.3f;
		
		}
		
	}
	
}
