using UnityEngine;
using System.Collections;

public class Particle : MonoBehaviour {
	
	public tk2dAnimatedSprite sprite;
	
	public float lifetime = 0f;
	public bool dieOnGroundHit = true;
	public bool gravity = true;
	public Vector3 velocity = Vector3.zero;
	public string spawnParticleOnDeath = "";
	public bool batch = false;
	public Color color = Color.blue;
	
	private Transform _transform;
	private Vector3 _position;
	private float time = 0f;
	private Color particleColor = Color.blue;
	
	public float x {
		get {
			return _position.x;
		}
		set {
			_position.Set(value, _position.y, _position.z);
		}
	}
	
	public float y {
		get {
			return _position.y;
		}
		set {
			_position.Set(_position.x, value, _position.z);
		}
	}
	
	void Awake()
	{
		particleColor = sprite.color;
	}
	
	// Use this for initialization
	void Start()
	{
		_transform = this.transform;
		_position = _transform.position;	
	}
	
	// Update is called once per frame
	void Update()
	{
		
		if (!this.enabled) {
			return;
		}
		
		if (lifetime > 0) {
			
			time += Time.deltaTime;
			
			if (lifetime - time <= 1f) {
				Color temp = sprite.color;
				temp.a = Mathf.Lerp(0f, particleColor.a, Mathf.Clamp01(lifetime - time));
				sprite.color = temp;
			}
			
			if (time > lifetime) {
				Die();
			}
			
		}
		
		CheckPhysics();
		SnapToGrid();
		
	}
	
	public void Reset()
	{
		this.enabled = true;
		time = 0;
		sprite.color = particleColor;
		gameObject.SetActiveRecursively(true);
		
		if (_transform != null) {
			_position = _transform.position;
		}
		
	}
	
	public void Die()
	{
		this.enabled = false;
		gameObject.SetActiveRecursively(false);
		
		if (spawnParticleOnDeath != "") {
			
			SnapToGrid(); //Make sure we're aligned to the grid
			
			if (ParticleManager.Instance == null) {
				Debug.Log("Particle Manager is NULL!?!");
			} else {	
				ParticleManager.Instance.SpawnParticle(_transform.position, spawnParticleOnDeath);
			}
			
		}
		
	}
	
	void CheckPhysics()
	{
		
		bool grounded = IsGrounded();
		
		//If We're Not Grounded 
		if (!grounded || velocity.y > 0f || !dieOnGroundHit) {
			
			this.y += velocity.y * Time.deltaTime;
			
			if (gravity) {
				velocity.y += Game.gravity.y * Time.deltaTime;
			}
			
		} else {
			velocity.y = 0f;
			if (dieOnGroundHit && grounded) {
				Die();
			} else if (grounded) {
				this.enabled = false;
			}
		}
		
		if (velocity.x != 0f) {
			this.x += velocity.x * Time.deltaTime;
		}
		
		if (this.y < -50) {
			Die();
		}

	}
	
	public void SnapToGrid()
	{
	
		Vector3 pt = _position;
		pt.x = Mathf.Round(pt.x);
		pt.y = Mathf.Round(pt.y);
		_transform.position = pt;
		
	}
	
	bool IsGrounded()
	{
		
		LayerMask mask = 1 << LayerMask.NameToLayer("Default");
		
		RaycastHit hitInfo;
		
		//Raycast to check for ground hit
		if (Physics.Raycast(_transform.position, Vector3.down, out hitInfo, 3.0f, mask.value)) {
			
			//Snap to ground if falling
			if (velocity.y <= 0) {
				velocity.y = 0;
				this.y = hitInfo.point.y;
			}
			
			return true;
			
		}
		
		return false;
		
	}
	
}
