using UnityEngine;
using System.Collections;

public class ParticleManager : MonoBehaviour {
	
	private static ParticleManager _instance;
	public static ParticleManager Instance {
		get {
			return _instance;
		}
	}
	
	public Particle paintParticle;
	public Particle paintPool;
	
	private int poolCount = 75;
	private tk2dStaticSpriteBatcher spriteBatcher;
	
	public Queue paintParticleInstances;
	public Queue paintPoolInstances;
	
	void Awake()
	{
		_instance = this;
		
		paintParticleInstances = new Queue();
		paintPoolInstances = new Queue();
		
		spriteBatcher = gameObject.GetComponent<tk2dStaticSpriteBatcher>();
		
	}
	
	// Use this for initialization
	void Start()
	{
		CreateInstances();
	}
	
	public Particle GetPaintParticle()
	{
		Particle particle = paintParticleInstances.Dequeue() as Particle;
		paintParticleInstances.Enqueue(particle);
		particle.Reset();
		return particle;
	}
	
	public Particle GetPaintPool()
	{
		Particle particle = paintPoolInstances.Dequeue() as Particle;
		paintPoolInstances.Enqueue(particle);
		particle.Reset();
		return particle;
	}
	
	public Particle SpawnParticle(Vector3 pt, string type)
	{
		
		if (type.ToLower() == "paintpool") {
			
			Particle paintPool = GetPaintPool() as Particle;
			
			Vector3 pos = paintPool.transform.position;
			pos.x = pt.x;
			pos.y = pt.y;
			paintPool.transform.position = pos;
			
			paintPool.Reset();
			
			if (paintPool.batch) {
				paintPool.gameObject.isStatic = true;
				paintPool.transform.parent = this.transform;
			} else {
				paintPool.transform.parent = null;
			}
			spriteBatcher.Build();
			
			return paintPool;
			
		} else if (type.ToLower() == "paintparticle") {
			
			Particle paintParticle = GetPaintParticle();
			
			Vector3 pos = paintParticle.transform.position;
			pos.x = pt.x;
			pos.y = pt.y;
			paintParticle.transform.position = pos;
			
			paintParticle.Reset();
			
			return paintParticle;
		
		} else {
		
			return null;
			
		}
		
	}
	
	void CreateInstances()
	{
		
		for (int i = 0; i < poolCount; i++) {
			
			Particle particle = Instantiate(paintParticle) as Particle;
			particle.enabled = false;
			particle.gameObject.SetActiveRecursively(false);
			paintParticleInstances.Enqueue(particle);
			
			particle = Instantiate(paintPool) as Particle;
			particle.enabled = false;
			particle.gameObject.SetActiveRecursively(false);
			paintPoolInstances.Enqueue(particle);
			
		}
		
	}
	
}
